
/*
 * restore.c  Phantasia Ultimate routines for restores
 */

#include "include.h"

extern int server_hook;

/************************************************************************
/
/ FUNCTION NAME: Do_restoreoptions(struct client_t *c)
/
/ FUNCTION: wizard administrative powers - restore options
/
/ AUTHOR: Samuel T,    8/20/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: A set of restore options
/
*************************************************************************/

Do_restoreoptions(struct client_t *c)
{
char error_msg[SZ_ERROR_MESSAGE];
char string_buffer[SZ_LINE], string_buffer2[SZ_LINE];
struct event_t *event_ptr;
struct button_t theButtons;
long answer;              /* pointer to option description */

	/* determine what the player wants to do */
    
    Do_clear_buttons(&theButtons, 0);

    strcpy(theButtons.button[0], "Natural\n");
    strcpy(theButtons.button[1], "Equip\n");
    strcpy(theButtons.button[2], "Currency\n");
    strcpy(theButtons.button[3], "Age\n");
    strcpy(theButtons.button[4], "Items\n");
    strcpy(theButtons.button[5], "Experience\n");
    strcpy(theButtons.button[6], "Restore Char\n");
    strcpy(theButtons.button[7], "Cancel\n");

    Do_send_line(c, "Which restore options?\n");
    
    if (Do_buttons(c, &answer, &theButtons) != S_NORM || answer == 7) {
        Do_send_clear(c);
	return;
    }

    Do_send_clear(c);
    
    switch(answer) {

        /* Natural Stats restore */
    case 0:
	
	Do_naturalrestore(c);

        return;

    case 1:

	Do_equiprestore(c);

	return;

    case 2:

        Do_currencyrestore(c);

        return;

    case 3:

	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = AGING_EVENT;
        strcpy(string_buffer2, "age to");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much age would you like to increase?\n")) {

	    free((void *)event_ptr);
	    return;
	}


	 /* if we're here, we have an event to send to another player */
	        /* prompt for player to affect */
        
             sprintf(string_buffer, "Who do you wish to send the %s?\n", string_buffer2);

	    if (Do_player_dialog(c, string_buffer, string_buffer2) != S_NORM) {
	        free((void *)event_ptr);
	        return;
	    }

	    event_ptr->arg4 = (void *) Do_malloc(strlen(c->modifiedName) + 1);
	    strcpy(event_ptr->arg4, c->modifiedName);
	    event_ptr->from = c->game;

	    if (!Do_send_character_event(c, event_ptr, string_buffer2)) {
	        free((void *)event_ptr->arg4);
	        free((void *)event_ptr);
	        Do_send_line(c, "That character just left the game.\n");
	        Do_more(c);
	        return;
	    }
	
	    Do_send_line(c, "Age restore complete.\n");
	    Do_more(c);
	    Do_send_clear(c);
	
	    return;

    case 4:

	Do_itemsrestore(c);

       return;

    case 5:
        event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = EXPERIENCE_EVENT;
        strcpy(string_buffer2, "experience to");

	if (Do_double_dialog(c, &event_ptr->arg1, "How much experience would you like to increase?\n")) {

	    free((void *)event_ptr);
	    return;
	}


	 /* if we're here, we have an event to send to another player */
	        /* prompt for player to affect */
        
             sprintf(string_buffer, "Who do you wish to send the %s?\n", string_buffer2);

	    if (Do_player_dialog(c, string_buffer, string_buffer2) != S_NORM) {
	        free((void *)event_ptr);
	        return;
	    }

	    event_ptr->arg4 = (void *) Do_malloc(strlen(c->modifiedName) + 1);
	    strcpy(event_ptr->arg4, c->modifiedName);
	    event_ptr->from = c->game;

	    if (!Do_send_character_event(c, event_ptr, string_buffer2)) {
	        free((void *)event_ptr->arg4);
	        free((void *)event_ptr);
	        Do_send_line(c, "That character just left the game.\n");
	        Do_more(c);
	        return;
	    }
	
	    Do_send_line(c, "Experience restore complete.\n");
	    Do_more(c);
	    Do_send_clear(c);
	
	    return;
    
    
    case 6:
    	Do_restore_character(c);
	return;
    
    default:
        sprintf(error_msg, "[%s] Returned non-option in Do_restoreoptions.\n", c->connection_id);
 
        Do_log_error(error_msg);
	Do_caught_hack(c, H_SYSTEM);
	return;
    }

    return;
}

/************************************************************************
/
/ FUNCTION NAME: Do_naturalrestore(struct client_t *c)
/
/ FUNCTION: Natural Restore Options
/
/ AUTHOR: Samuel T, 8/20/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Restore Natural Stats
/
*************************************************************************/

Do_naturalrestore(struct client_t *c)
{
struct event_t *event_ptr;
struct button_t theButtons;
char string_buffer[SZ_LINE], string_buffer2[SZ_LINE];
char error_msg[SZ_ERROR_MESSAGE];
long answer;              /* pointer to option description */

    Do_clear_buttons(&theButtons, 0);

	/* determine what the supreme wizard wants to increase */
    strcpy(theButtons.button[0], "Energy\n");
    strcpy(theButtons.button[1], "Strength\n");
    strcpy(theButtons.button[2], "Speed\n");
    strcpy(theButtons.button[3], "Magic Lvl\n");
    strcpy(theButtons.button[4], "Brains\n");
    strcpy(theButtons.button[5], "Lives\n");
    strcpy(theButtons.button[6], "Poison\n");
    strcpy(theButtons.button[7], "Cancel\n");

    Do_send_line(c, "What natural restore command do you wish to use?\n");
    
    if (Do_buttons(c, &answer, &theButtons) != S_NORM || answer == 7) {
	Do_send_clear(c);
	return;
    }
    Do_send_clear(c);
    
    switch(answer) {

    case 0:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = NATINC_EVENT;
        event_ptr->arg1 = 0;
        strcpy(string_buffer2, "base energy");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for base energy?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;

    case 1:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = NATINC_EVENT;
        event_ptr->arg1 = 1;
        strcpy(string_buffer2, "base strength");
        
        if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for base strength?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;                        

    case 2:
        event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = NATINC_EVENT;
        event_ptr->arg1 = 2;
        strcpy(string_buffer2, "base speed");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for base speed?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;

    case 3:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = NATINC_EVENT;
        event_ptr->arg1 = 3;
        strcpy(string_buffer2, "magic level");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for magic level?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;
    case 4:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = NATINC_EVENT;
        event_ptr->arg1 = 4;
        strcpy(string_buffer2, "brains");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for brains?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;

    case 5:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = NATINC_EVENT;
        event_ptr->arg1 = 5; 
        strcpy(string_buffer2, "lives");        
  
	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for lives?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;
        
    case 6:

	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = NATINC_EVENT;
        event_ptr->arg1 = 6; 
        strcpy(string_buffer2, "poison");        
                
	if (Do_long_dialog(c, &event_ptr->arg3, "How much poison would you like to increase?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }
    
    break;

    default:
        sprintf(error_msg, "[%s] Returned non-option in Do_naturalrestore.\n",
                c->connection_id);
 
        Do_log_error(error_msg);
	Do_caught_hack(c, H_SYSTEM);
	return;
    }

	/* if we're here, we have an event to send to another player */
        /* prompt for player to affect */
    sprintf(string_buffer, "Whose %s do you wish to increase?\n", string_buffer2);
    if (Do_player_dialog(c, string_buffer, string_buffer2) != S_NORM) {
        free((void *)event_ptr);
        return;
    }

    event_ptr->arg4 = (void *) Do_malloc(strlen(c->modifiedName) + 1);
    strcpy(event_ptr->arg4, c->modifiedName);
    event_ptr->from = c->game;

    if (!Do_send_character_event(c, event_ptr, string_buffer2)) {
        free((void *)event_ptr->arg4);
        free((void *)event_ptr);
        Do_send_line(c, "That character just left the game.\n");
        Do_more(c);
        return;
    }

    Do_send_line(c, "Natural restore complete.\n");
    Do_more(c);
    Do_send_clear(c);
    return;
}

/************************************************************************
/
/ FUNCTION NAME: Do_natincreceive(struct client_t *c, struct event_t *the_event)
/
/ FUNCTION: do the natural increase 
/
/ AUTHOR: Samuel T, 8/22/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Increases the natural stats set up by the supreme wizard
/
*************************************************************************/

Do_natincreceive(struct client_t *c, struct event_t *the_event)
{
    char string_buffer[SZ_LINE];
    double dtemp;
    long answer;
    long increase;

    answer = the_event->arg1;
    increase = the_event->arg3;
/*
    sprintf(string_buffer, "%s's casts increase of %.0lf!\n", increase);
    Do_send_line(c, string_buffer);
  */  
    switch(answer) {    
        
    case 0:
    	Do_energy(c, c->player.max_energy + increase + c->player.shield,
	          c->player.max_energy + increase, c->player.shield, 0, FALSE);
	
    break;    

    case 1:
    	Do_strength(c, c->player.max_strength + increase,
			  c->player.sword, 0, FALSE);

    break;    
    
    case 2:
    	 Do_speed(c, c->player.max_quickness + increase, c->player.quicksilver, 0, FALSE);
    	
    break;    
    
    case 3:
    	
    	c->player.magiclvl = c->player.magiclvl + increase;    	
    	
    break;    
    
    case 4:
    	
    	c->player.brains = c->player.brains + increase;  	 
    	
    break;    
    
    case 5:
    	
    	c->player.lives = c->player.lives + increase;
    	
    	if (c->wizard < 4){
    		if (c->player.lives > 5){
 	    		c->player.lives = 5;
    		}
    	}
    	
    break;    
    
    
    case 6: 
	    c->player.poison = c->player.poison + increase;

    break;
    
    }
    
    sprintf(string_buffer, "%s's casts a natural restore spell on you, you feel different!\n", 
           the_event->arg4);

    Do_send_line(c, string_buffer);

    sprintf(string_buffer, "%s casted a restore spell on %s.\n", the_event->arg4, c->player.lcname);
    
    Do_log(NATRESTORE_LOG, &string_buffer);

    free((void *)the_event->arg4);   

    Do_more(c);
    Do_send_clear(c);

    return;
}


/************************************************************************
/
/ FUNCTION NAME: Do_equiprestore(struct client_t *c)
/
/ FUNCTION: Equipment Restore Options
/
/ AUTHOR: Samuel T, 8/21/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Restore Equipment
/
*************************************************************************/

Do_equiprestore(struct client_t *c)
{
struct event_t *event_ptr;
struct button_t theButtons;
char string_buffer[SZ_LINE], string_buffer2[SZ_LINE];
char error_msg[SZ_ERROR_MESSAGE];
long answer;              /* pointer to option description */

    Do_clear_buttons(&theButtons, 0);

	/* determine what the supreme wizard wants to increase */
    strcpy(theButtons.button[0], "Shield\n");
    strcpy(theButtons.button[1], "Sword\n");
    strcpy(theButtons.button[2], "Quicksilver\n");
    strcpy(theButtons.button[3], "Charms\n");
    strcpy(theButtons.button[4], "Amulets\n");
    strcpy(theButtons.button[5], "Holy Water\n");
    strcpy(theButtons.button[6], "Crowns\n");
    strcpy(theButtons.button[7], "Cancel\n");

    Do_send_line(c, "What equipment restore command do you wish to use?\n");
    
    if (Do_buttons(c, &answer, &theButtons) != S_NORM || answer == 7) {
	Do_send_clear(c);
	return;
    }
    Do_send_clear(c);
    
    switch(answer) {

    case 0:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = EQINC_EVENT;
        event_ptr->arg1 = 0;
        strcpy(string_buffer2, "shield");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for shield?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;

    case 1:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = EQINC_EVENT;
        event_ptr->arg1 = 1;
        strcpy(string_buffer2, "sword");
        
        if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for sword?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;                        

    case 2:
        event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = EQINC_EVENT;
        event_ptr->arg1 = 2;
        strcpy(string_buffer2, "quicksilver");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for quicksilver?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;

    case 3:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = EQINC_EVENT;
        event_ptr->arg1 = 3;
        strcpy(string_buffer2, "charms");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for charms?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;

    case 4:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = EQINC_EVENT;
        event_ptr->arg1 = 4;
        strcpy(string_buffer2, "amulets");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for amulets?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;

    case 5:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = EQINC_EVENT;
        event_ptr->arg1 = 5; 
        strcpy(string_buffer2, "holy water");        
  
	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for holy water?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;
        
    case 6:

	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = EQINC_EVENT;
        event_ptr->arg1 = 6; 
        strcpy(string_buffer2, "crowns");        
                
	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for crowns?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }
    
    break;

    default:
        sprintf(error_msg, "[%s] Returned non-option in Do_equiprestore.\n", c->connection_id);
 
        Do_log_error(error_msg);
	Do_caught_hack(c, H_SYSTEM);
	return;
    }

	/* if we're here, we have an event to send to another player */
        /* prompt for player to affect */
    sprintf(string_buffer, "Whose %s do you wish to increase?\n", string_buffer2);
    if (Do_player_dialog(c, string_buffer, string_buffer2) != S_NORM) {
        free((void *)event_ptr);
        return;
    }

    event_ptr->arg4 = (void *) Do_malloc(strlen(c->modifiedName) + 1);
    strcpy(event_ptr->arg4, c->modifiedName);
    event_ptr->from = c->game;

    if (!Do_send_character_event(c, event_ptr, string_buffer2)) {
        free((void *)event_ptr->arg4);
        free((void *)event_ptr);
        Do_send_line(c, "That character just left the game.\n");
        Do_more(c);
        return;
    }

    Do_send_line(c, "Equipment restore complete.\n");
    Do_more(c);
    Do_send_clear(c);
    return;
}

/************************************************************************
/
/ FUNCTION NAME: Do_eqincreceive(struct client_t *c, struct event_t *the_event)
/
/ FUNCTION: do the equipment increase 
/
/ AUTHOR: Samuel T, 8/22/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Increases the equipment set up by the supreme wizard
/
*************************************************************************/

Do_eqincreceive(struct client_t *c, struct event_t *the_event)
{
    char string_buffer[SZ_LINE];
    double dtemp;
    long answer;
    long increase;

    answer = the_event->arg1;
    increase = the_event->arg3;
 
    switch(answer) {    
        
    case 0:
    	Do_energy(c, c->player.max_energy + c->player.shield + increase,
	          c->player.max_energy, c->player.shield + increase, 0, FALSE);
	
    break;    

    case 1:
    	Do_strength(c, c->player.max_strength, c->player.sword + increase, 0, FALSE);

    break;    
    
    case 2:
    	 Do_speed(c, c->player.max_quickness, c->player.quicksilver + increase, 0, FALSE);
    	
    break;    
    
    case 3:
    	
    	c->player.charms = c->player.charms + increase;    	
    	
    break;    
    
    case 4:
    	
    	c->player.amulets = c->player.amulets + increase;  	 
    	
    break;    
    
    case 5:
    	
    	c->player.holywater = c->player.holywater + increase;
    	    	    	
    break;    
        
    case 6: 
	    Do_crowns(c, increase, TRUE);

    break;
    
    }
    
    sprintf(string_buffer, "%s's casts an equipment spell on you, you feel different!\n", 
           the_event->arg4);

    Do_send_line(c, string_buffer);

    sprintf(string_buffer, "%s casted an equipment spell on %s.\n", the_event->arg4, c->player.lcname);
    
    Do_log(EQRESTORE_LOG, &string_buffer);

    free((void *)the_event->arg4);   

    Do_more(c);
    Do_send_clear(c);

    return;
}

/************************************************************************
/
/ FUNCTION NAME: Do_currencyrestore(struct client_t *c)
/
/ FUNCTION: Currency Restore Options
/
/ AUTHOR: Samuel T, 8/22/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Restore Currency
/
*************************************************************************/

Do_currencyrestore(struct client_t *c)
{
struct event_t *event_ptr;
struct button_t theButtons;
char string_buffer[SZ_LINE], string_buffer2[SZ_LINE];
char error_msg[SZ_ERROR_MESSAGE];
long answer;              /* pointer to option description */

    Do_clear_buttons(&theButtons, 0);

	/* determine what the supreme wizard wants to increase */
    strcpy(theButtons.button[0], "Gems\n");
    strcpy(theButtons.button[1], "Gold\n");
    strcpy(theButtons.button[7], "Cancel\n");

    Do_send_line(c, "What currency restore command do you wish to use?\n");
    
    if (Do_buttons(c, &answer, &theButtons) != S_NORM || answer == 7) {
	Do_send_clear(c);
	return;
    }
    Do_send_clear(c);
    
    switch(answer) {

    case 0:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = CINC_EVENT;
        event_ptr->arg1 = 0;
        strcpy(string_buffer2, "gems");

	if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for gems?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;

    case 1:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = CINC_EVENT;
        event_ptr->arg1 = 1;
        strcpy(string_buffer2, "gold");
        
        if (Do_long_dialog(c, &event_ptr->arg3, "How much would you like to increase for gold?\n")) {

	    free((void *)event_ptr);
	    return;
	}

	if (event_ptr->arg3 <= 0){
		free((void *)event_ptr);
	    return;
        }	

        break;                        


    default:
        sprintf(error_msg, "[%s] Returned non-option in Do_currencyrestore.\n", c->connection_id);
 
        Do_log_error(error_msg);
	Do_caught_hack(c, H_SYSTEM);
	return;
    }

	/* if we're here, we have an event to send to another player */
        /* prompt for player to affect */
    sprintf(string_buffer, "Whose %s do you want to increase?\n", string_buffer2);
    if (Do_player_dialog(c, string_buffer, string_buffer2) != S_NORM) {
        free((void *)event_ptr);
        return;
    }

    event_ptr->arg4 = (void *) Do_malloc(strlen(c->modifiedName) + 1);
    strcpy(event_ptr->arg4, c->modifiedName);
    event_ptr->from = c->game;

    if (!Do_send_character_event(c, event_ptr, string_buffer2)) {
        free((void *)event_ptr->arg4);
        free((void *)event_ptr);
        Do_send_line(c, "That character just left the game.\n");
        Do_more(c);
        return;
    }

    Do_send_line(c, "Currency restore complete.\n");
    Do_more(c);
    Do_send_clear(c);
    return;
}


/************************************************************************
/
/ FUNCTION NAME: Do_cincreceive(struct client_t *c, struct event_t *the_event)
/
/ FUNCTION: do the currency increase 
/
/ AUTHOR: Samuel T,	8/22/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Increases the currency set up by the supreme wizard
/
*************************************************************************/

Do_cincreceive(struct client_t *c, struct event_t *the_event)
{
    char string_buffer[SZ_LINE];
    double dtemp;
    long answer;
    long increase;

    answer = the_event->arg1;
    increase = the_event->arg3;
 
    switch(answer) {    
        
    case 0:
    	c->player.gems = c->player.gems + increase; 
    	Do_check_weight(c);

	Do_send_int(c, GEMS_PACKET);
        Do_send_double(c, c->player.gems);
    break;    

    case 1:
		c->player.gold = c->player.gold + increase;				
		Do_check_weight(c);
		
		Do_send_int(c, GOLD_PACKET);
	        Do_send_double(c, c->player.gold);
    break;               
    
    }
    
    sprintf(string_buffer, "%s's casts a currency spell on you, you feel heavier!\n", 
           the_event->arg4);

    Do_send_line(c, string_buffer);

    sprintf(string_buffer, "%s casted an currency spell on %s.\n", the_event->arg4, c->player.lcname);
    
    Do_log(CRESTORE_LOG, &string_buffer);

    free((void *)the_event->arg4);   

    Do_more(c);
    Do_send_clear(c);

    return;
}

/************************************************************************
/
/ FUNCTION NAME: Do_ageincreceive(struct client_t *c, struct event_t *the_event)
/
/ FUNCTION: do the age increase 
/
/ AUTHOR: Samuel T,  8/22/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Increases the age set up by the supreme wizard
/
*************************************************************************/

Do_ageincreceive(struct client_t *c, struct event_t *the_event)
{
    char string_buffer[SZ_LINE];
    double dtemp;
    long increase;

    increase = the_event->arg3;
 
    c->player.age = c->player.age + increase;	       
    
    sprintf(string_buffer, "%s's casts an aging spell on you, you feel older!\n", 
           the_event->arg4);

    Do_send_line(c, string_buffer);

    sprintf(string_buffer, "%s casted an aging spell on %s.\n", the_event->arg4, c->player.lcname);
    
    Do_log(AGERESTORE_LOG, &string_buffer);

    free((void *)the_event->arg4);   

    Do_more(c);
    Do_send_clear(c);

    return;
}

/************************************************************************
/
/ FUNCTION NAME: Do_itemsrestore(struct client_t *c)
/
/ FUNCTION: Item Restore Options
/
/ AUTHOR: Samuel T, 8/22/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Restore Items
/
*************************************************************************/

Do_itemsrestore(struct client_t *c)
{
struct event_t *event_ptr;
struct button_t theButtons;
char string_buffer[SZ_LINE], string_buffer2[SZ_LINE];
char error_msg[SZ_ERROR_MESSAGE];
long answer;              /* pointer to option description */

    Do_clear_buttons(&theButtons, 0);

	/* determine what the supreme wizard wants to give */
    strcpy(theButtons.button[0], "Bless\n");
    strcpy(theButtons.button[1], "Palintir\n");
    strcpy(theButtons.button[2], "Virgin\n");
    strcpy(theButtons.button[3], "Normal Ring\n");
    strcpy(theButtons.button[4], "The One Ring\n");
    Do_clear_buttons(&theButtons, 5);
    Do_clear_buttons(&theButtons, 6);
    strcpy(theButtons.button[7], "Cancel\n");

    Do_send_line(c, "What item restore command do you wish to use?\n");
    
    if (Do_buttons(c, &answer, &theButtons) != S_NORM || answer == 7) {
	Do_send_clear(c);
	return;
    }
    Do_send_clear(c);
    
    switch(answer) {

    case 0:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = ITEMINC_EVENT;
        event_ptr->arg1 = 0;
        strcpy(string_buffer2, "bless");
		
        break;

    case 1:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = ITEMINC_EVENT;
        event_ptr->arg1 = 1;
        strcpy(string_buffer2, "palintir");
        
        break;                        

    case 2:
        event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = ITEMINC_EVENT;
        event_ptr->arg1 = 2;
        strcpy(string_buffer2, "virgin");

        break;

    case 3:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = ITEMINC_EVENT;
        event_ptr->arg1 = 3;
        strcpy(string_buffer2, "normal ring");

        break;

    case 4:
	event_ptr = (struct event_t *) Do_create_event();
        event_ptr->type = ITEMINC_EVENT;
        event_ptr->arg1 = 4;
        strcpy(string_buffer2, "The One Ring");	
        
        break;

            
    
    default:
        sprintf(error_msg, "[%s] Returned non-option in Do_itemsrestore.\n",
                c->connection_id);
 
        Do_log_error(error_msg);
	Do_caught_hack(c, H_SYSTEM);
	return;
    }

	/* if we're here, we have an event to send to another player */
        /* prompt for player to affect */
    sprintf(string_buffer, "Who do you wish to give a %s to?\n", string_buffer2);
    if (Do_player_dialog(c, string_buffer, string_buffer2) != S_NORM) {
        free((void *)event_ptr);
        return;
    }

    event_ptr->arg4 = (void *) Do_malloc(strlen(c->modifiedName) + 1);
    strcpy(event_ptr->arg4, c->modifiedName);
    event_ptr->from = c->game;

    if (!Do_send_character_event(c, event_ptr, string_buffer2)) {
        free((void *)event_ptr->arg4);
        free((void *)event_ptr);
        Do_send_line(c, "That character just left the game.\n");
        Do_more(c);
        return;
    }

    Do_send_line(c, "Item restore complete.\n");
    Do_more(c);
    Do_send_clear(c);
    return;
}

/************************************************************************
/
/ FUNCTION NAME: Do_itemincreceive(struct client_t *c, struct event_t *the_event)
/
/ FUNCTION: do the currency increase 
/
/ AUTHOR: Samuel T,	8/22/02
/
/ ARGUMENTS: none
/
/ RETURN VALUE: none
/
/ MODULES CALLED: writerecord(), adjuststats(), fabs(), more(), sqrt(),
/       sleep(), floor(), wmove(), drandom(), wclear(), printw(),
/       altercoordinates(), infloat(), waddstr(), wrefresh(), mvprintw(), getanswer(),
/       wclrtoeol(), wclrtobot()
/
/ DESCRIPTION: Increases the currency set up by the supreme wizard
/
*************************************************************************/

Do_itemincreceive(struct client_t *c, struct event_t *the_event)
{
    char string_buffer[SZ_LINE];
    double dtemp;
    long answer;

    answer = the_event->arg1;
    
    switch(answer) {    
        
    case 0:	
	 Do_award_blessing(c);    	
    break;    

    case 1:	    	    
	    if (!c->player.palantir){
		    Do_palantir(c, TRUE, FALSE);	    
	    }	    
    break;               
    
    case 2:
	    Do_virgin(c, TRUE, FALSE);
    break;

    case 3:
           if (c->player.ring_type == R_NONE){
               Do_ring(c, R_NAZREG, FALSE);    
	   }
    break;

    case 4:    
	    if (c->player.ring_type != R_NONE){
	    	    Do_ring(c, R_NONE, FALSE);
            }
            Do_ring(c, R_DLREG, FALSE);
    break;

}

    
    sprintf(string_buffer, "%s's casts an item spell on you. You leap for joy!!!\n", 
           the_event->arg4);

    Do_send_line(c, string_buffer);

    sprintf(string_buffer, "%s casted an item spell on %s.\n", the_event->arg4, c->player.lcname);
    
    Do_log(ITEMRESTORE_LOG, &string_buffer);

    free((void *)the_event->arg4);   

    Do_more(c);
    Do_send_clear(c);

    return;
}
